@@ -6,11 +6,11 @@ from typing import Union
 from dateutil.parser import ParserError
 import dateutil.parser as dparse
 
+
 import gantt_project_maker.gantt as gantt
 from gantt_project_maker.colors import color_to_hex
 from gantt_project_maker.excelwriter import write_planning_to_excel
 
-
 SCALES = dict(
     daily=gantt.DRAW_WITH_DAILY_SCALE,
     weekly=gantt.DRAW_WITH_WEEKLY_SCALE,
@@ -23,7 +23,7 @@ _logger = logging.getLogger(__name__)
 
 def get_nearest_saturday(date):
     """
-    Get the the nearest Saturday with respect to 'date'
+    Get the nearest Saturday with respect to 'date'
 
     Parameters
     ----------
@@ -67,9 +67,9 @@ def parse_date(
         Datum
     """
     if date_string is not None:
-        date = dparse.parse(date_string, dayfirst=dayfirst).date()
+        date = dparse.parse(date_string.strip(), dayfirst=dayfirst).date()
     elif date_default is not None and isinstance(date_default, str):
-        date = dparse.parse(date_default, dayfirst=dayfirst).date()
+        date = dparse.parse(date_default.strip(), dayfirst=dayfirst).date()
     else:
         date = date_default
     return date
@@ -144,10 +144,11 @@ class Vacation(StartEndBase):
 
 
 class Employee:
-    def __init__(self, label, full_name=None, vacations=None):
+    def __init__(self, label, full_name=None, vacations=None, color=None):
         self.label = label
         self.full_name = full_name
-        self.resource = gantt.Resource(name=label, fullname=full_name)
+        self.color = color
+        self.resource = gantt.Resource(name=label, fullname=full_name, color=color)
 
         if vacations is not None:
             self.vacations = add_vacation_employee(
@@ -164,6 +165,7 @@ class BasicElement(StartEndBase):
         start=None,
         dependent_of=None,
         color=None,
+        project_color=None,
         detail=False,
         display=True,
         dayfirst=False,
@@ -175,6 +177,7 @@ class BasicElement(StartEndBase):
         self.detail = detail
         self.dependent_of = dependent_of
         self.color = color_to_hex(color)
+        self.project_color = color_to_hex(project_color)
         self.display = display
 
 
@@ -188,6 +191,7 @@ class Task(BasicElement):
         employees=None,
         dependent_of=None,
         color=None,
+        project_color=None,
         detail=False,
         display=True,
         dayfirst=True,
@@ -197,17 +201,30 @@ class Task(BasicElement):
             start=start,
             dependent_of=dependent_of,
             color=color,
+            project_color=project_color,
             detail=detail,
             display=display,
             dayfirst=dayfirst,
         )
         self.end = parse_date(end, dayfirst=dayfirst)
         self.duration = duration
+        if self.end is None and self.duration is None:
+            msg = (
+                f"For a Task, next to a start date, either a end date or a duration needs to be specified. "
+                f"None is given for task '{label}'"
+            )
+            raise ValueError(msg)
+        if self.end is not None:
+            if self.end < self.start:
+                msg = f"End date {self.end} is before {self.start} for Task '{label}'. Please fix this"
+                raise ValueError(msg)
         self.employees = employees
 
         self.element = self.add_task()
 
     def add_task(self) -> gantt.Task:
+        if self.color is None:
+            self.color = self.project_color
         task = gantt.Task(
             name=self.label,
             start=self.start,
@@ -227,6 +244,7 @@ class Milestone(BasicElement):
         start=None,
         dependent_of=None,
         color=None,
+        project_color=None,
         detail=False,
         display=True,
         dayfirst=True,
@@ -236,6 +254,7 @@ class Milestone(BasicElement):
             start=start,
             dependent_of=dependent_of,
             color=color,
+            project_color=project_color,
             detail=detail,
             display=display,
             dayfirst=dayfirst,
@@ -257,22 +276,31 @@ class ProjectPlanner:
     def __init__(
         self,
         programma_title: str = None,
+        vacations_title: str = None,
         programma_color: str = None,
-        output_file_name: str = None,
+        vacation_color: str = None,
+        output_file_name: Path = None,
+        vacation_output_file_name: Path = None,
         planning_start: datetime = None,
         planning_end: datetime = None,
+        weeks_margin_left: int = None,
+        weeks_margin_right: int = None,
         today: datetime = None,
         dayfirst: bool = False,
         scale: str = None,
         period_info: dict = None,
         excel_info: dict = None,
         details: bool = None,
+        filter_employees: list = None,
+        save_svg_as_pdf: bool = False,
     ):
         """
 
         Args:
             programma_title: str
                 Main titel of the whole project
+            vacations_title: str
+                Titel of the vacations project
             programma_color: str
                 First color of the bar
             output_file_name: str
@@ -281,6 +309,10 @@ class ProjectPlanner:
                 Start of the program
             planning_end: datetime
                 End of the program
+            weeks_margin_left: int
+                Shift the end of the planning so many weeks to the left without adding projects
+            weeks_margin_right: int
+                Shift the end of the planning so many weeks to the right without adding projects
             today: datetime
                 Today's date
             dayfirst: bool
@@ -290,9 +322,11 @@ class ProjectPlanner:
             period_info: dict
                Information on the periods output
             excel_info: dict
-                Information on the excel output
+                Information on the Excel output
             details: bool
                 If true, include the details to the programs
+            filter_employees: list
+                If not None, only add task to which  employees in this list contribute
         """
         self.period_info = period_info
         self.planning_start = planning_start
@@ -301,6 +335,15 @@ class ProjectPlanner:
         self.dayfirst = dayfirst
         self.scale = scale
         self.details = details
+        self.save_svg_as_pdf = save_svg_as_pdf
+        if filter_employees:
+            # if filter_employees are given, add them as a set
+            self.filter_employees = set(filter_employees)
+        else:
+            self.filter_employees = None
+
+        self.weeks_margin_left = weeks_margin_left
+        self.weeks_margin_right = weeks_margin_right
 
         self.excel_info = excel_info
 
@@ -309,11 +352,16 @@ class ProjectPlanner:
         else:
             self.output_file_name = Path(output_file_name)
 
-        # het hoofdproject maken we alvast aan.
+        # Make the main project
         self.programma = gantt.Project(
             name=programma_title, color=color_to_hex(programma_color)
         )
 
+        # Make the project to store all the vacations
+        self.vacations_gantt = gantt.Project(
+            name=vacations_title, color=color_to_hex(vacation_color)
+        )
+
         self.project_tasks = dict()
         self.vacations = dict()
         self.employees = dict()
@@ -382,7 +430,13 @@ class ProjectPlanner:
                 depends_of = self.subprojects[key]
                 _logger.debug(f"Dependent of project: {key}")
             except KeyError:
-                raise AssertionError(f"Dependency {key} does not exist")
+                msg = f"Dependency {key} does not exist"
+                if self.filter_employees is not None:
+                    # in case we are filtering on employees, some dependencies may be missing. Just give a warning
+                    _logger.warning(msg)
+                    depends_of = None
+                else:
+                    raise AssertionError(msg)
 
         return depends_of
 
@@ -470,14 +524,47 @@ class ProjectPlanner:
         _logger.info("Adding employees...")
         for w_key, w_prop in employees_info.items():
             _logger.debug(f"Adding {w_key} ({w_prop.get('name')})")
+            full_name = w_prop.get("name")
+            employee_vacations_info = w_prop.get("vacations")
+            employee_color = w_prop.get("color")
+
             self.employees[w_key] = Employee(
                 label=w_key,
-                full_name=w_prop.get("name"),
-                vacations=w_prop.get("vacations"),
+                full_name=full_name,
+                color=employee_color,
+                vacations=employee_vacations_info,
             )
 
+            # also add the vacations of this employee to the vacation gantt project in order to give an overview later
+            employee_vacations = gantt.Project(
+                name=full_name,
+                color=employee_color,
+                font=gantt.get_font_attributes(font_weight="bold", font_size="20"),
+            )
+            if employee_vacations_info is not None:
+                for v_key, v_prop in employee_vacations_info.items():
+                    vacation_name = v_prop.get("label", v_key)
+                    vacation_start = v_prop.get("start")
+                    vacation_end = v_prop.get("end")
+                    if vacation_end is None:
+                        vacation_duration = 1
+                    else:
+                        vacation_duration = None
+                    vacation_task = Task(
+                        label=vacation_name,
+                        color=v_prop.get("color"),
+                        start=vacation_start,
+                        end=vacation_end,
+                        duration=vacation_duration,
+                        dayfirst=self.dayfirst,
+                    )
+                    employee_vacations.add_task(vacation_task.element)
+            self.vacations_gantt.add_task(employee_vacations)
+
     def make_task_of_milestone(
-        self, task_properties: dict = None
+        self,
+        task_properties: dict = None,
+        project_color=None,
     ) -> Union[Task, Milestone]:
         """
         Add all the general tasks and milestones
@@ -486,6 +573,8 @@ class ProjectPlanner:
         ----------
         task_properties: dict
             Dictionary with tasks or milestones
+        project_color: str
+            Color of the parent project
 
         Returns
         -------
@@ -503,6 +592,7 @@ class ProjectPlanner:
                 end=task_properties.get("end"),
                 duration=task_properties.get("duration"),
                 color=task_properties.get("color"),
+                project_color=project_color,
                 detail=task_properties.get("detail", False),
                 employees=employees,
                 dependent_of=dependencies,
@@ -514,6 +604,7 @@ class ProjectPlanner:
                 label=task_properties.get("label"),
                 start=task_properties.get("start"),
                 color=task_properties.get("color"),
+                project_color=project_color,
                 dependent_of=dependencies,
                 dayfirst=self.dayfirst,
             )
@@ -559,6 +650,17 @@ class ProjectPlanner:
                         msg = f"De task key {task_key} has been used before. Please pick another name"
                         _logger.warning(msg)
                         raise ValueError(msg)
+                    if self.filter_employees is not None:
+                        contributors = task_val.get("employees")
+                        is_contributing = check_if_employee_in_contributing(
+                            filter_employees=self.filter_employees,
+                            contributing_employees=contributors,
+                        )
+                        if not is_contributing:
+                            _logger.debug(
+                                f"None of {contributors} are in {self.filter_employees}. Skipping"
+                            )
+                            continue
                     tasks_en_mp[task_key] = tasks_and_milestones_info[module_key][
                         task_key
                     ]
@@ -582,13 +684,18 @@ class ProjectPlanner:
         Make all projects
         """
         employee_color = color_to_hex(subprojects_color)
-        projects_employee = gantt.Project(name=subprojects_title, color=employee_color)
+
+        projects_employee = gantt.Project(
+            name=subprojects_title,
+            color=employee_color,
+            font=gantt.get_font_attributes(font_weight="bold", font_size="20"),
+        )
 
         _logger.info(f"Add all projects of {subprojects_title}")
         for project_key, project_values in subprojects_info.items():
-            _logger.info(f"Making project: {project_values['title']}")
+            project_name = project_values.get("title", project_key)
 
-            project_name = project_values["title"]
+            _logger.info(f"Making project: {project_name}")
 
             project_color = color_to_hex(project_values.get("color"))
 
@@ -640,13 +747,40 @@ class ProjectPlanner:
                                 # de task een ander project?
                                 task = self.subprojects[task_val]
                             except KeyError as err:
-                                _logger.warning(f"{err}")
-                                raise
+                                if not self.filter_employees:
+                                    _logger.warning(f"{err}")
+                                    raise
+                                else:
+                                    _logger.debug(f"{err}")
+                                    continue
                     else:
-                        task_obj = self.make_task_of_milestone(task_properties=task_val)
+                        task_obj = self.make_task_of_milestone(
+                            task_properties=task_val, project_color=project_color
+                        )
                         task = task_obj.element
                         is_detail = task_obj.detail
 
+                    if task.color is None:
+                        task.color = project_color
+
+                    if self.filter_employees:
+                        try:
+                            contributors = task.employees
+                        except AttributeError:
+                            _logger.debug(
+                                "No employees found, but can be a projects, so just add"
+                            )
+                        else:
+                            is_contributing = check_if_employee_in_contributing(
+                                filter_employees=self.filter_employees,
+                                contributing_employees=contributors,
+                            )
+                            if not is_contributing:
+                                _logger.debug(
+                                    f"None of {contributors} are in {self.filter_employees}. Skipping"
+                                )
+                                continue
+
                     if not self.details and is_detail:
                         _logger.debug(f"skipping task {task_key} as it is a detail")
                     else:
@@ -663,7 +797,9 @@ class ProjectPlanner:
         self,
         planning_output_directory,
         resource_output_directory,
+        vacations_output_directory,
         write_resources=False,
+        write_vacations=False,
         periods=None,
     ):
         """
@@ -673,16 +809,19 @@ class ProjectPlanner:
         ----------
         write_resources: bool
             Write the resources file as well (next to the gantt charts which is always written)
+        write_vacations: bool
+            Write the vacations file as well
         planning_output_directory: Path
-            Output directory van de svg files van de planning
+            Output directory of the svg files of the planning
         resource_output_directory: Path
-            Output directory van de svg files van de resources
+            Output directory of the svg files of the resources
+        vacations_output_directory: Path
+            Output directory of the svg files of the vacations
         periods: list
-            List of periods we want to add. If None, add all periods
+           Periods we want to add. If None, add all periods
         """
 
         for period_key, period_prop in self.period_info.items():
-
             if periods is not None and period_key not in periods:
                 _logger.debug(f"Employee {period_key} is skipped")
                 continue
@@ -693,6 +832,8 @@ class ProjectPlanner:
             )
             file_base_resources = file_base_tasks.replace("_tasks", "_resources")
 
+            file_base_vacations = file_base_tasks.replace("_tasks", "_vacations")
+
             planning_output_directory.mkdir(exist_ok=True, parents=True)
 
             if write_resources:
@@ -705,6 +846,17 @@ class ProjectPlanner:
                 file_base_resources
             ).with_suffix(suffix)
 
+            file_name_vac = vacations_output_directory / Path(
+                file_base_vacations
+            ).with_suffix(suffix)
+
+            weeks_margin_left = period_prop.get(
+                "weeks_margin_left", self.weeks_margin_left
+            )
+            weeks_margin_right = period_prop.get(
+                "weeks_margin_right", self.weeks_margin_right
+            )
+
             scale = period_prop.get("scale")
             if scale is not None:
                 scale = SCALES[scale]
@@ -721,6 +873,7 @@ class ProjectPlanner:
                 self.planning_end,
                 dayfirst=self.dayfirst,
             )
+
             today = parse_date(
                 period_prop.get("today"), self.date_today, dayfirst=self.dayfirst
             )
@@ -739,23 +892,125 @@ class ProjectPlanner:
                 f"Writing project starting at {start} and ending at {end} with a scale {scale} to {file_name}"
             )
             self.programma.make_svg_for_tasks(
-                filename=file_name.as_posix(),
+                filename=file_name,
                 start=start,
                 end=end,
+                margin_left=weeks_margin_left,
+                margin_right=weeks_margin_right,
                 scale=scale,
                 today=today,
             )
             _logger.debug("Done")
 
+            if self.save_svg_as_pdf:
+                try:
+                    import svg42pdf
+                except ImportError as err:
+                    _logger.warning(f"{err}\nFailed writing pdf because svg42pdf is")
+                else:
+                    pdf_file_name = file_name.with_suffix(".pdf")
+                    svg42pdf.svg42pdf(
+                        svg_fn=file_name.as_posix(),
+                        pdf_fn=pdf_file_name.as_posix(),
+                        method="any",
+                    )
+
             if write_resources:
                 _logger.info(
                     f"Write resources of {start} to {end} with scale {scale} to {file_name_res}"
                 )
-                self.programma.make_svg_for_resources(
-                    filename=file_name_res.as_posix(),
+                try:
+                    self.programma.make_svg_for_resources(
+                        filename=file_name_res.as_posix(),
+                        start=start,
+                        end=end,
+                        scale=SCALES["daily"],
+                        today=today,
+                    )
+                except TypeError as err:
+                    _logger.warning(err)
+                    _logger.warning(
+                        "Failed writing the resources with the above error message. Please check your "
+                        "employee's input data"
+                    )
+                else:
+                    if self.save_svg_as_pdf and svg42pdf is not None:
+                        pdf_file_name_res = file_name_res.with_suffix(".pdf")
+                        svg42pdf.svg42pdf(
+                            svg_fn=file_name_res.as_posix(),
+                            pdf_fn=pdf_file_name_res.as_posix(),
+                            method="any",
+                        )
+
+            write_vacations_this_period = period_prop.get("export_vacations", False)
+            if write_vacations or write_vacations_this_period:
+                vacations_output_directory.mkdir(exist_ok=True, parents=True)
+                _logger.info(f"Writing vacation file {file_name_vac}")
+                self.vacations_gantt.make_svg_for_tasks(
+                    filename=file_name_vac,
                     start=start,
                     end=end,
+                    margin_left=weeks_margin_left,
+                    margin_right=weeks_margin_right,
                     scale=scale,
                     today=today,
                 )
+                if self.save_svg_as_pdf and svg42pdf is not None:
+                    pdf_file_name_vac = file_name_vac.with_suffix(".pdf")
+                    svg42pdf.svg42pdf(
+                        svg_fn=file_name_vac.as_posix(),
+                        pdf_fn=pdf_file_name_vac.as_posix(),
+                        method="any",
+                    )
+
             _logger.debug("Done")
+
+
+def check_if_employee_in_contributing(
+    filter_employees: list, contributing_employees: list
+) -> bool:
+    """
+    Check if any of the employees given in filter_employees is in contributing to this taks
+    """
+
+    is_contributing = False
+
+    if contributing_employees is not None:
+        if isinstance(contributing_employees, str):
+            contributing_employees = set(list([contributing_employees]))
+        else:
+            contributing_employees = set(contributing_employees)
+        if contributing_employees.intersection(filter_employees):
+            _logger.debug(
+                f"Contributing employees {contributing_employees} not in filter list"
+            )
+            is_contributing = True
+
+    return is_contributing
+
+
+def extend_suffix(output_filename: Path, extensions: Union[list, str]):
+    """
+    Add an extra suffix to the base filename
+
+    Parameters
+    ----------
+    output_filename: Path
+        Base filename
+    extensions: str or list
+        Extra suffixes to add
+
+    Returns
+    -------
+    Path:
+        New filename with extra suffix
+
+    """
+    suffix = output_filename.suffix
+    if isinstance(extensions, str):
+        extensions = [extensions]
+    output_filename = Path(
+        "_".join([output_filename.with_suffix("").as_posix()] + extensions)
+    ).with_suffix(suffix)
+
+    return output_filename
